"use strict";
/*
 * Copyright (c) 2022.
 * Author Peter Placzek (tada5hi)
 * For the full copyright and license information,
 * view the LICENSE file that was distributed with this source code.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupHTTPMiddleware = void 0;
const core_1 = require("@typescript-auth/core");
const http_1 = require("@typescript-error/http");
const domains_1 = require("@typescript-auth/domains");
const utils_1 = require("../../utils");
function setupHTTPMiddleware(context) {
    const tokenCache = (0, utils_1.initTokenCache)(context.redis, context.redisPrefix);
    const tokenAPIClient = new domains_1.TokenAPI(context.axios);
    const cookieHandler = (cookies) => {
        if (cookies === null || cookies === void 0 ? void 0 : cookies.auth_token) {
            const { access_token: accessToken } = JSON.parse(cookies === null || cookies === void 0 ? void 0 : cookies.auth_token);
            return accessToken;
        }
        return undefined;
    };
    return (req, res, next) => __awaiter(this, void 0, void 0, function* () {
        let { authorization: headerValue } = req.headers;
        if (!headerValue) {
            try {
                let value;
                if (context.cookieHandler) {
                    value = context.cookieHandler(req.cookies);
                }
                else {
                    value = cookieHandler(req.cookies);
                }
                if (value) {
                    headerValue = (0, core_1.stringifyAuthorizationHeader)({ type: 'Bearer', token: value });
                }
            }
            catch (e) {
                // ...
            }
        }
        if (!headerValue) {
            next();
            return;
        }
        const header = (0, core_1.parseAuthorizationHeader)(headerValue);
        if (header.type !== 'Bearer') {
            throw new http_1.BadRequestError('Only Bearer tokens are accepted as authentication method.');
        }
        let data;
        try {
            data = yield (0, utils_1.verifyToken)({
                token: header.token,
                tokenCache,
                tokenAPIClient,
            });
        }
        catch (e) {
            next(e);
            return;
        }
        const _a = data.entity.data, { permissions } = _a, entity = __rest(_a, ["permissions"]);
        switch (data.entity.type) {
            case 'robot':
                req.robotId = entity.id;
                req.robot = entity;
                break;
            case 'user':
                req.userId = entity.id;
                req.user = entity;
                break;
        }
        req.token = header.token;
        req.permissions = permissions;
        req.ability = new core_1.AbilityManager(permissions);
        next();
    });
}
exports.setupHTTPMiddleware = setupHTTPMiddleware;
//# sourceMappingURL=module.js.map